package com.szyx.tcm.supervision.config;

import com.fiftyonetrust.common.model.exception.CodeException;
import com.szyx.tcm.supervision.enums.ResultEnum;
import com.szyx.tcm.supervision.util.SZYXAccessUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/*******************************************************************
 * <pre></pre>
 * @文件名称： InsuranceInterceptor.java
 * @包 路  径： com.szyx.tcm.supervision.config
 * @Copyright：wy (C) 2024 *
 * @Description: 医保系统拦截器 只用满足鉴权的请求才能访问
 * @Version: V1.0
 * @Author： wy
 * @Date： 2024/7/26 16:13
 * @Modify：
 */
@Slf4j
@Component
public class InsuranceInterceptor  implements HandlerInterceptor {


    public static void main(String[] args) {
        long timestamp = System.currentTimeMillis();
        System.out.println(SZYXAccessUtil.encryptTimeStamp(timestamp));
        System.out.println(SZYXAccessUtil.encryptByAES(Long.toString(timestamp)));
    }

    @Value("${insurance.host}")
    private String targetUrl;



    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {

        String requestUrl = request.getRequestURL().toString();
        // 医保回调的直接转发
        if (requestUrl.startsWith(targetUrl)) {
            // If the request URL starts with targetUrl, bypass the interceptor
            return true;
        }

        // 其他请求一律进行请求头校验
        String timeStamp = request.getHeader("timestamp");
        String accessToken = request.getHeader("accessToken");

        if ( StringUtils.isBlank(timeStamp) || StringUtils.isBlank(accessToken) ) {
            throw new CodeException(ResultEnum.VERIFY_ERROR.getCode(), ResultEnum.VERIFY_ERROR.getMessage());
        }

        if(!SZYXAccessUtil.verifyTimeStamp(timeStamp, accessToken)){
            throw new CodeException(ResultEnum.VERIFY_ERROR.getCode(), ResultEnum.VERIFY_ERROR.getMessage());
        }


        return HandlerInterceptor.super.preHandle(request, response, handler);
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {
        HandlerInterceptor.super.postHandle(request, response, handler, modelAndView);
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
        HandlerInterceptor.super.afterCompletion(request, response, handler, ex);
    }
}
