package com.szyx.tcm.supervision.controller.insurance;

import com.fiftyonetrust.common.model.result.R;
import com.szyx.tcm.supervision.model.dto.insurance.*;
import com.szyx.tcm.supervision.model.dto.insurance.core.Signinoutb;
import com.szyx.tcm.supervision.service.manager.HealthInsuranceManager;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/*******************************************************************
 * <pre></pre>
 * @文件名称： HealthInsurenceController.java
 * @包 路  径： com.szyx.tcm.web.patient.controller.insurance
 * @Copyright：wy (C) 2024 *
 * @Description:
 * @Version: V1.0
 * @Author： wy
 * @Date： 2024/7/16 10:12
 * @Modify：
 */
@RestController
@RequestMapping("/insurance")
public class HealthInsuranceController {

    @Resource
    private HealthInsuranceManager healthInsuranceManager;


    // 1 根据购药单id生成小程序医保授权链接
    @GetMapping("/getAuthUrl/{platformOrderId}")
    @ApiOperation(value = "生成小程序医保授权链接")
    public R<String> getAuthUrl(@PathVariable("platformOrderId") String medicineOrderId) {
        return R.ok(healthInsuranceManager.getAuthUrl(medicineOrderId));
    }


    // 2 河南医保平台下单，获取订单医保分账

    @PostMapping("/getOrder")
    @ApiOperation(value = "河南医保平台下单，获取订单医保分账")
    public R<InsurancePayOrderOutputMessage> getOrder(@RequestBody InsuranceGetOrderReq req) {
        return R.ok(healthInsuranceManager.getOrder(req));
    }

    // 3 根据医保平台下单分账信息，确认支付，微信统一下单
    @PostMapping("/confirmPay")
    @ApiOperation(value = "根据医保平台下单分账信息，确认支付，微信统一下单")
    public R<UnifiedOrderRep> confirmPay(@RequestBody UnifiedOrderReq req) {
        return R.ok(healthInsuranceManager.confirmPay(req));
    }



    // 根据授权码查询用户的医保电子凭证信息
    @PostMapping("/getInsuranceInfo")
    @ApiOperation(value = "根据授权码查询用户的医保电子凭证信息")
    public R<InsuranceRep> getInsuranceInfo(@RequestBody InsuranceReq req) {
        return R.ok(healthInsuranceManager.getInsuranceInfo(req));
    }

    // 省医保核心系统「9001」 接口签到获取流水号 ，后续接口调用前需先调用此接口获取流水号
    @PostMapping("/getSerialNumber")
    @ApiOperation(value = "接口签到获取流水号")
    public R getSerialNumber() {
        return R.ok(healthInsuranceManager.getSerialNumber());
    }


    // 省医保核心系统「1101」获取医保人员基础信息
    @PostMapping("/getBasicInsuranceInfo")
    @ApiOperation(value = "获取医保人员基础信息")
    public R getBasicInsuranceInfo() {
        return R.ok(healthInsuranceManager.getBasicInsuranceInfo(new Signinoutb()));
    }


    // 省医保平台「6201」上传处方药费明细
    @PostMapping("/uploadRecipeDetail")
    @ApiOperation(value = "上传处方药费明细")
    public R uploadRecipeDetail() {
        return R.ok(healthInsuranceManager.uploadRecipeDetail(new InsuranceGetOrderReq(), new InsuranceRep(), new InsuranceCoreOutputMessage1101()));
    }


    // 省医保平台「6202」支付下单
    @PostMapping("/payOrder")
    @ApiOperation(value = "支付下单")
    public R payOrder() {
        return R.ok(healthInsuranceManager.payOrder(new InsuranceGetOrderReq() , new InsuranceRep(), new InsuranceUploadRecipeDetailOutputMessage()));
    }


    //获取微信医保移动支付access_token 服务商获取方式
    @PostMapping("/getInsuranceAccessToken")
    @ApiOperation(value = "获取微信医保移动支付access_token")
    public R<InsuranceAccessTokenRep> getInsuranceAccessToken(@RequestBody InsuranceAccessTokenReq req) {
        return R.ok(healthInsuranceManager.InsuranceAccessToken(req));
    }


    // 微信医保平台统一下单
    @PostMapping("/unifiedOrder")
    @ApiOperation(value = "统一下单")
    public R unifiedOrder() throws Exception {
        return R.ok(healthInsuranceManager.unifiedOrder(new UnifiedOrderReq()));
    }


    // 读取当前配置信息
    @GetMapping("/getInsuranceConfig")
    @ApiOperation(value = "读取当前配置信息")
    public R getInsuranceConfig() {
        return R.ok(healthInsuranceManager.getInsuranceConfig());
    }


}
