package com.szyx.tcm.supervision.controller.insurance;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import java.util.Enumeration;
import java.util.Objects;

/*******************************************************************
 * <pre></pre>
 * @文件名称： ProxyController.java
 * @包 路  径： com.szyx.tcm.supervision.controller.insurance
 * @Copyright：wy (C) 2024 *
 * @Description:   用于进行代理转发 所有从云端系统过来的请求都会经过这个controller
 * @Version: V1.0
 * @Author： wy
 * @Date： 2024/7/26 17:34
 * @Modify：
 */
@RestController
@RequestMapping("/proxy")
@Slf4j
public class ProxyController {

    @Value("${insurance.host}")
    private String targetUrl;

    @Value("${server.servlet.context-path}")
    private String contextPath;


    private final RestTemplate restTemplate;


    public ProxyController(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }


    @RequestMapping("/**")
    public ResponseEntity<String> proxyRequest(HttpServletRequest request, @RequestBody(required = false) String body) {
        String url = targetUrl + request.getRequestURI().substring(("/" + contextPath + "/proxy").length());

        HttpHeaders headers = new HttpHeaders();
        Enumeration<String> headerNames = request.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String headerName = headerNames.nextElement();
            headers.add(headerName, request.getHeader(headerName));
        }

        HttpHeaders responseHeaders = new HttpHeaders();
        responseHeaders.setContentType(MediaType.APPLICATION_JSON);

        HttpEntity<String> entity = new HttpEntity<>(body, headers);
        ResponseEntity<String> response = null;
        try {
            log.info("Proxy request: {} {}", request.getMethod(), url);
            log.info("Proxy request body: {}", body);
            response = restTemplate.exchange(url, Objects.requireNonNull(HttpMethod.resolve(request.getMethod())), entity, String.class);
        } catch (HttpClientErrorException e) {
            String resBody = e.getResponseBodyAsString();
            log.error("Proxy request failed: {} {}", e.getStatusCode(), resBody);
            return new ResponseEntity<>(resBody, responseHeaders, e.getStatusCode());
        }

        return new ResponseEntity<>(response.getBody(), responseHeaders,response.getStatusCode());
    }



}
