package com.szyx.tcm.supervision.service.manager;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONWriter;
import com.szyx.tcm.supervision.constant.WeChatInsurancePayConstant;
import com.szyx.tcm.supervision.model.dto.insurance.*;
import com.szyx.tcm.supervision.model.dto.insurance.core.SignIn;
import com.szyx.tcm.supervision.model.dto.insurance.core.Signinoutb;
import com.szyx.tcm.supervision.util.DateUtils;
import com.szyx.tcm.supervision.util.HttpUtil;
import com.szyx.tcm.supervision.util.InsuranceWechatUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.*;
import java.math.BigDecimal;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.*;

import static com.szyx.tcm.supervision.constant.WeChatInsurancePayConstant.*;


/*******************************************************************
 * <pre></pre>
 * @文件名称： HealthInsuranceManager.java
 * @包 路  径： com.szyx.tcm.web.patient.service.manager
 * @Copyright：wy (C) 2024 *
 * @Description:
 * @Version: V1.0
 * @Author： wy
 * @Date： 2024/7/16 10:37
 * @Modify：
 */
@Slf4j
@Service
//@RefreshScope
public class HealthInsuranceManager {

//    @Resource
//    private PatientUserMapper patientUserMapper;

//    @Resource
//    private MedicineOrderMapper medicineOrderMapper;

//    @Resource
//    private InquiryService inquiryService;

    //    获取信息服务接口 测试环境
    // todo 合作方id 密钥
    String getInsuranceAPI = "https://mip-receiver.tengmed.com/testapi/mipuserquery/userQuery/%s";

    //   获取支付access_token
    // 调用医保支付平台的API（即https://api.weixin.qq.com/payinsurance/前缀的API）需要使用单独的key 进行MD5签名
    String getInsuranceAccess = "https://api.weixin.qq.com/payinsurance/gettoken?grant_type=%s&agentappid=%s&agentsecret=%s&appid=%s";


    // 微信医保平台统一下单API接口
    String unifiedOrderAPI = "https://api.weixin.qq.com/payinsurance/unifiedorder?access_token=%s";

    // 获取授权码
    String weChatPayAPI = "auth/pages/bindcard/auth/index?openType=getAuthCode&cityCode=%s&channel=%s&redirectUrl=%s";

    // 合作方 id
//    @Value("${insurance.partnerId}")
    private String partnerId;


    // 合作方密钥
//    @Value("${insurance.partnerSecret}")
    private String partnerSecret;

    // 代理帐号id
//    @Value("${insurance.agentappid}")
    private String agentappid;

    // 代理帐号appsecret
//    @Value("${insurance.agentsecret}")
    private String agentsecret;


    // 医院小程序帐号id
//    @Value("${insurance.appid}")
    private String appid;

    /**
     * 医院小程序商户号
     */
//    @Value("${insurance.mchId}")
    private String mchId;

    /**
     * 服务商商户号
     */
//    @Value("${insurance.subMchId}")
    private String subMchId;

    // 医保区划
//    @Value("${insurance.cityCode}")
    private String cityCode;

    /**
     * 腾讯医保城市ID
     */
//    @Value("${insurance.cityId}")
    private String cityId;

    // 渠道号（微信医保平台分配）
//    @Value("${insurance.channel}")
    private String channel;

    // 回跳地址
//    @Value("${insurance.redirectUrl}")
    private String redirectUrl;

    // 机构渠道认证编码 小程序授权不需要encode
//    @Value("${insurance.orgChnlCrtfCodg}")
    private String orgChnlCrtfCodg;

    // 定点医药机构编码 来源于国家局医保移动支付反馈单
//    @Value("${insurance.orgCodg}")
    private String orgCodg;

    // 线上核验业务类型编码  医保移动支付业务填04107
//    @Value("${insurance.bizType}")
    private String bizType;

    // orgAppId 定点医药机构应用ID 来源于国家局医保移动支付反馈单
//    @Value("${insurance.orgAppId}")
    private String orgAppId;


    // 微信医保平台 获取支付授权码
//    @Value("${insurance.getAuthUrl}")
    private String authUrlPath;


    // 9001 测试接口地址
//    @Value("${insurance.getSerialNumber}")
    private String getSerialNumberPath;

    // 1101 测试接口地址
//    @Value("${insurance.getBasicInsuranceInfo}")
    private String getBasicInsuranceInfoPath;

    // 6201 测试接口地址
//    @Value("${insurance.uploadRecipeDetail}")
    private String uploadRecipeDetailPath;

    // 6202 测试接口地址
//    @Value("${insurance.payOrder}")
    private String payOrderPath;


    /**
     * 医保定点医药机构编号
     */
//    @Value("${insurance.fixmedinsCode}")
    private String fixmedinsCode;

    /**
     * 医保定点医药机构名称
     */
//    @Value("${insurance.fixmedinsName}")
    private String fixmedinsName;


    /**
     * 就医地医保区划
     */
//    @Value("${insurance.mdtrtareaAdmvs}")
    private String mdtrtareaAdmvs;

    /**
     * 接收方系统代码
     */
//    @Value("${insurance.recerSysCode}")
    private String recerSysCode;


    /**
     * 测试就诊凭证编号
     */
//    @Value("${insurance.mdtrtCertNo}")
    private String mdtrtCertNo;

    /**
     * 测试就诊凭证姓名
     */
//    @Value("${insurance.mdtrtCertName}")
    private String mdtrtCertName;


    private final static char[] hexArray = "0123456789abcdef".toCharArray();


    /**
     * 生成小程序医保授权链接
     *
     * @param medicineOrderId 购药单id
     * @return 授权链接
     */
    public String getAuthUrl(String medicineOrderId) {
        return String.format(authUrlPath, cityCode, channel, orgChnlCrtfCodg, orgCodg, bizType, orgAppId);
    }


    /**
     * 获取微信医保移动支付access_token
     *
     * @param req 请求参数
     * @return 返回结果
     */
    @SneakyThrows
    public InsuranceAccessTokenRep InsuranceAccessToken(InsuranceAccessTokenReq req) {
        String pathUrl = String.format(getInsuranceAccess, "client_credential", agentappid, agentsecret, appid);
        String s = postData("payinsurance", "gettoken", pathUrl, partnerId, req.toString());
        log.info("InsuranceAccessToken: {}", s);
        return JSON.parseObject(s, InsuranceAccessTokenRep.class);
    }

    /**
     * 根据医保平台下单分账信息，确认支付，微信统一下单
     * @return
     */
    @SneakyThrows
    public UnifiedOrderRep confirmPay(UnifiedOrderReq req){
        // 获取订单信息  todo
        req.setOpenId("");

        // 获取AccessToken
        InsuranceAccessTokenRep accessToken = InsuranceAccessToken(new InsuranceAccessTokenReq());
        req.setAccessToken(accessToken);
        // 微信统一下单
        return unifiedOrder( req);

    }


    /**
     * 河南医保平台下单，获取订单医保分账
     *
     * @return 分账数据
     */
    public InsurancePayOrderOutputMessage getOrder(InsuranceGetOrderReq req) {

//        //读取当前订单信息 todo
//        if (req.getOrderType() == 1){
//            Inquiry inquiry = inquiryService.getInquiryByRecipeId(Integer.parseInt(req.getPlatformOrderId()));
//            req.setOrgBizSer(inquiry.getBizOrderNo());
//            req.setChrgBchno(inquiry.getBizOrderNo());
//            req.setMedOrgOrd(inquiry.getBizOrderNo());
//            req.setFeeType(INSURANCE_PAY_FEE_TYPE);
//            req.setMdtrtId(inquiry.getId().toString());
//
//        }








        //微信获取用户信息服务接口
        InsuranceRep insuranceInfo = getInsuranceInfo(new InsuranceReq().setQrcode(req.getAuthCode()));
        // 获取用户在河南医保平台当地的医保信息「1101」
        InsuranceCoreOutputMessage1101 insuranceCoreInfo = getInsuranceInfo();

        // 省医保上传
        InsuranceUploadRecipeDetailOutputMessage uploadDetail = new InsuranceUploadRecipeDetailOutputMessage();
        if(req.getOrderType() == 1){
            uploadDetail = uploadInquiryDetail(req, insuranceInfo, insuranceCoreInfo);
        }else{
             uploadDetail = uploadRecipeDetail(req, insuranceInfo, insuranceCoreInfo);
        }


        // 省医保下单
        InsurancePayOrderOutputMessage paidOrder = payOrder(req, insuranceInfo, uploadDetail);

        //     追加医保结构体 用于微信统一下单  requestContent;
        Map<String, String> requestContent = new HashMap<>();
        requestContent.put("payOrdId", paidOrder.getPayOrdId());
        requestContent.put("payAuthNo", req.getAuthCode());
        requestContent.put("setlLatlnt", insuranceInfo.getUserLongitudeLatitude().getLatitude() + "," + insuranceInfo.getUserLongitudeLatitude().getLongitude());


        String requestContentStr = JSON.toJSONString(requestContent);
        paidOrder.setRequestContent(requestContentStr);

        //返回分账
        InsurancePayOrderOutputMessage outputMessage = new InsurancePayOrderOutputMessage();
        // todo
        outputMessage.setPayOrdId("123456");
        outputMessage.setOrdStas("1");
        outputMessage.setFeeSumamt(new BigDecimal("0.01"));
        outputMessage.setOwnPayAmt(new BigDecimal("0.01"));
        outputMessage.setPsnAcctPay(new BigDecimal("0.01"));
        outputMessage.setFundPay(new BigDecimal("0.01"));


        return outputMessage;
    }


    // 获取用户在河南医保平台当地的医保信息
    public InsuranceCoreOutputMessage1101 getInsuranceInfo() {
        // 9001 签到
        Signinoutb serialNumber = getSerialNumber();

        // 1101 获取医保人员基础信息
        return getBasicInsuranceInfo(serialNumber);
    }


    /**
     * 省医保平台「9001」 接口签到获取流水号 ，后续接口调用前需先调用此接口获取流水号
     *
     * @return
     */
    public Signinoutb getSerialNumber() {

        // ===========参数梳理 ===========
        // infno 交易编号
        // msgid 发送方报文ID
        // mdtrtarea_admvs 就医地医保区划
        // recer_sys_code 接收方系统代码
        // infver 接口版本号
        // opter_type 操作员类型 1-经办人；2-自助终端；3-移动终端
        // opter 操作员

        InsuranceCoreCommonInputMessage inputMessage = new InsuranceCoreCommonInputMessage();
        SignIn signIn = new SignIn();
        inputMessage.setInfno(WeChatInsurancePayConstant.INSURANCE_CORE_MSG_ID_SIGN_IN);
        inputMessage.setMsgid(InsuranceWechatUtil.getMsgId(fixmedinsCode));
        inputMessage.setMdtrtareaAdmvs(mdtrtareaAdmvs);
        inputMessage.setRecerSysCode(recerSysCode);
        inputMessage.setInfver(INSURANCE_CORE_SYSTEM_VERSION);
        inputMessage.setOpterType("3");
        inputMessage.setOpter("wy");
        inputMessage.setInfTime(DateUtils.format(new Date(), DateUtils.PATTERN_DATE_TIME));
        inputMessage.setFixmedinsCode(fixmedinsCode);
        inputMessage.setFixmedinsName(fixmedinsName);
        inputMessage.setInput(JSONObject.toJSONString(signIn));
        signIn.setOpterNo("wy");
        signIn.setMac("00-00-00-00-00-00");
        signIn.setIp("127.0.0.1");


        String result = HttpUtil.postByJson(getSerialNumberPath, JSONObject.toJSONString(inputMessage));
        log.info("getSerialNumber: {}", result);

        InsuranceCoreCommonOutputMessage outputMessage = JSON.parseObject(result, InsuranceCoreCommonOutputMessage.class);

        if (outputMessage.getInfcode() != null && outputMessage.getInfcode() == 0) {
            String output = outputMessage.getOutput();
            return JSON.parseObject(output, Signinoutb.class);
        }

        return null;
    }

    /**
     * 省医保核心系统「1101」获取医保人员基础信息
     *
     * @return
     */
    public InsuranceCoreOutputMessage1101 getBasicInsuranceInfo(Signinoutb serialNumber) {

        // ===========参数梳理 ===========
        // mdtrt_cert_type 就诊凭证类型
        // mdtrt_cert_no 就诊凭证编号 todo 测试使用 电子凭证令牌 还是 身份证号
        // begntime 开始时间
        // psn_cert_type 人员证件类型  使用电子凭证必填
        // certno 证件号码
        // psn_name 人员姓名

        // =============== 响应参数 ===============
        // baseinfo 基础信息
        //      -psn_no 人员编号
        //      -psn_cert_type 人员证件类型
        //      -certno 证件号码
        //      -psn_name 人员姓名
        //      -gend 性别
        //      -naty 民族
        //      -brdy 出生日期
        //      -age 年龄
        // insuinfo
        //      -balc 余额
        //      -insutype 险种类型
        //      -psn_type 人员类型
        //      -psn_insu_stas 人员参保状态
        //      -psn_insu_date 个人参保日期
        //      -paus_insu_date 暂停参保日期
        //      -cvlserv_flag 公务员标志
        //      -insuplc_admdvs 参保地医保区划
        //      -emp_name 单位名称
        // idetinfo
        //      -psn_idet_type 识别类型
        //      -psn_type_lv 人员类别级别
        //      -
        InsuranceCoreCommonInputMessage inputMessage = new InsuranceCoreCommonInputMessage();
        Map<String, InsuranceCoreInputMessage1101> map = new HashMap<>();
        InsuranceCoreInputMessage1101 inputMessage1101 = new InsuranceCoreInputMessage1101();
        map.put("data", inputMessage1101);
        inputMessage.setInfno(INSURANCE_CORE_BASIC_INFO);
        inputMessage.setMsgid(InsuranceWechatUtil.getMsgId(fixmedinsCode));
        inputMessage.setMdtrtareaAdmvs(mdtrtareaAdmvs);
        inputMessage.setRecerSysCode(recerSysCode);
        inputMessage.setInfver(INSURANCE_CORE_SYSTEM_VERSION);
        inputMessage.setOpterType("3");
        inputMessage.setOpter("wy");
        inputMessage.setInfTime(DateUtils.format(new Date(), DateUtils.PATTERN_DATE_TIME));
        inputMessage.setFixmedinsCode(fixmedinsCode);
        inputMessage.setFixmedinsName(fixmedinsName);
        inputMessage.setSignNo(serialNumber.getSignNo());
        inputMessage.setInput(JSONObject.toJSONString(map));
        inputMessage1101.setMdtrtCertType("02");
        // 明文身份证号
        inputMessage1101.setMdtrtCertNo(mdtrtCertNo);
        inputMessage1101.setPsnName(mdtrtCertName);

        String result = HttpUtil.postByJson(getBasicInsuranceInfoPath, JSONObject.toJSONString(inputMessage));
        log.info("getBasicInsuranceInfo: {}", result);

        InsuranceCoreCommonOutputMessage outputMessage = JSON.parseObject(result, InsuranceCoreCommonOutputMessage.class);

        if (outputMessage.getInfcode() != null && outputMessage.getInfcode() == 0) {
            String output = outputMessage.getOutput();

            return JSON.parseObject(output, InsuranceCoreOutputMessage1101.class);
        }
        return null;
    }

    /**
     * 省医保平台「6201」上传问诊费用明细
     *
     * @return
     */
    public InsuranceUploadRecipeDetailOutputMessage uploadInquiryDetail(InsuranceGetOrderReq req, InsuranceRep insuranceInfo, InsuranceCoreOutputMessage1101 insuranceCoreInfo) {


        // ============ 参数梳理 ===========

        // orgCodg 机构编码
        // orgId 电子凭证机构号
        // psnNo 人员编号 来源于 医保中心 医保核心系统 「1101」接口
        // insutype 险种类型 310:职工基本医疗保险 390:城乡居民基本医疗保险 320:公务员医疗补助 392:城乡居民大病保险 330:大额医疗费用补助 340:离休人员医疗保障 510:生育保险
        // medOrgOrd 医疗机构订单号  todo 使用药费订单号
        // rxCircFlag 电子处方流转标志  todo 确认是否需要
        // begntime 开始时间 挂号时间 格式： yyyy-MM-dd HH:mm:ss
        // idNo 患者证件号码
        // userName 用户姓名
        // idType 证件类别 居民身份证 01
        // ecToken 电子凭证授权
        // insuCode 就诊参保地行政区划  todo 确认是否需要 为空(null)时取用户电子凭证默认参保地
        // iptOtpNo 住院/门诊号  todo 确认是否可以用问诊单号
        // deptName 科室名称
        // caty 科别
        // medType 医疗类别 todo 确认互联网医院处方的医疗类别
        // feeType 费用类别  todo 确认互联网医院处方的费用类别 互联网医院问诊 05
        // medfeeSumamt 医疗费总额
        // acctUsedFlag 是否使用个人账户  todo 确认是否需要 药店上传费用时必填
        // diseCodg 疾病编码 todo 确认类型
        // diseName 疾病名称
        // psnSetlway 个人结算方式  todo 确认字典
        // chrgBchno 收费批次号
        // pubHospRfomFlag 公立医院改革标志

        // -diseinfoList 诊断或症状明细
        //      -diagType 诊断类型 1 西医主要诊断 2 西医其他诊断 3 中医主病诊断 4 中医主证诊断
        //      -diagSrtNo 诊断排序号
        //      -diagCode 诊断代码
        //      -diagName 诊断名称
        //      -diagDept 诊断科室
        //      -diseDorNo 诊断医生编码
        //      -diseDorName 诊断医生姓名
        //      -diagTime 诊断时间 yyyy-MM-dd HH:mm:ss
        //      -valiFlag 有效标志 0 无效 1 有效
        // -feedetailList 费用明细
        //      -feedetlSn 费用明细序号
        //      -psnNo 人员编号 来源于 医保中心 医保核心系统 「1101」接口
        //      -chrgBchno 收费批次号
        //      -diseCodg 疾病编码
        //      -rxno 处方号 todo 外购处方时，传入外购处方的处方号；非外购处方，传入医药机构处方号
        //      -rxCircFlag todo 外购处方标志 需要确认互联网医院的处方是否为外购处方
        //      -feeOcurTime 费用发生时间 yyyy-MM-dd HH:mm:ss
        //      -medListCodg 医疗目录编码 todo 确认互联网医院处方的医疗目录编码
        //      -medinsListCodg 医药机构目录编码 todo 确认互联网医院处方的医药机构目录编码
        //      -detItemFeeSumamt 明细项目费用总额
        //      -cnt 数量
        //      -pric 单价
        //      -bilgDeptCodg 开单科室编码
        //      -bilgDeptName 开单科室名称
        //      -bilgDrCodg 开单医生编码 todo 医生标准编码
        //      -bilgDrName 开单医师姓名
        //      -hospApprFlag 医院审批标志 1
        //      -medType 医疗类别 todo 确认互联网医院处方的医疗类别
        //      -medListName 医疗目录名称
        //      -medListSpc 医疗目录规格
        // payAuthNo 支付授权码
        // uldLatlnt 用户经纬度 定位失败情况下传0,0
        // mdtrtCertType 就诊凭证类型 02 身份证 01 电子凭证 todo 确认填身份证还是电子凭证


        // todo 校验必填项

        InsuranceUploadRecipeDetailInputMessage uploadData = new InsuranceUploadRecipeDetailInputMessage();
        // 医保核心系统患者基本信息
        InsuranceCoreOutputMessage1101.Baseinfo baseinfo = insuranceCoreInfo.getBaseinfo();
        //  医保核心系统保信息
        InsuranceCoreOutputMessage1101.Insuinfo insuinfo = insuranceCoreInfo.getInsuinfo();
        // 费用明细
        List<FeeDetail> feedetail = new ArrayList<>();
        // 诊断或状态明细
        List<DiseInfo> diseinfo = new ArrayList<>();


        uploadData.setOrgCodg(fixmedinsCode);
//        uploadData.setOrgId();
        uploadData.setPsnNo(baseinfo.getPsnNo());
        uploadData.setIdType(insuinfo.getInsutype());
        uploadData.setMedOrgOrd(req.getMedOrgOrd());
        // 电子处方流转标志  todo 目前河南省并未测试过电子处方线上医保支付
        uploadData.setRxCircFlag("0");
        uploadData.setBegntime(DateUtils.format(new Date(), DateUtils.PATTERN_DATE_TIME));
        uploadData.setIdNo(baseinfo.getCertno());
        uploadData.setUserName(baseinfo.getPsnName());
        uploadData.setIdType("01");
        uploadData.setEcToken(req.getAuthCode());
        uploadData.setInsuCode(mdtrtareaAdmvs);
        // 取问诊单id
        uploadData.setIptOtpNo(req.getPlatformOrderId());
        // 字典 dept
        uploadData.setDeptName("预防保健科");
        uploadData.setCaty("A01");
        uploadData.setMedType("91");
        uploadData.setFeeType(INSURANCE_PAY_FEE_TYPE);
        uploadData.setMedfeeSumamt(new BigDecimal("0.01"));
//        uploadData.setAcctUsedFlag("0");
        uploadData.setDiseCodg("A01");
        uploadData.setDiseName("感冒");
        uploadData.setPsnSetlway("01");
        // 公立医院内容 // 疑问点1 互联网医院是否为公立医院 公立医院改革标志 如何传参
        uploadData.setChrgBchno(req.getChrgBchno());
        uploadData.setDiseinfoList(diseinfo);
        uploadData.setFeedetailList(feedetail);


        // 组装诊断数据
        DiseInfo diseInfo = new DiseInfo();
        diseInfo.setDiagType("1");
        diseInfo.setDiagSrtNo(1);
        diseInfo.setDiagCode("A01");
        diseInfo.setDiagName("感冒");
        diseInfo.setDiagDept("预防保健科");
        // todo 确认医生编码
        diseInfo.setDiseDorNo("wy");
        diseInfo.setDiseDorName("wy");
        diseInfo.setDiagTime(DateUtils.format(new Date(), DateUtils.PATTERN_DATE_TIME));
        diseInfo.setValiFlag("1");


        // 组装费用数据
        FeeDetail feeDetail = new FeeDetail();
        feeDetail.setFeedetlSn(req.getOrgBizSer());
        feeDetail.setPsnNo(baseinfo.getPsnNo());
        feeDetail.setChrgBchno(req.getChrgBchno());
        feeDetail.setFeeOcurTime(DateUtils.format(new Date(), DateUtils.PATTERN_DATE_TIME));
        // todo 确定医疗目录 机构目录编码
        feeDetail.setMedListCodg("A01");
        feeDetail.setMedinsListCodg("A01");
        feeDetail.setHospApprFlag("1");
        feeDetail.setMedListName("感冒");
        feeDetail.setMedListSpc("感冒");
        // todo 医疗类别
        feeDetail.setMedType("91");
        feeDetail.setDetItemFeeSumamt(new BigDecimal("0.01"));
        feeDetail.setCnt(new BigDecimal("1"));
        feeDetail.setPric(new BigDecimal("0.01"));
        feeDetail.setBilgDeptCodg("A01");
        feeDetail.setBilgDeptName("预防保健科");
        // todo 医生编码
        feeDetail.setBilgDrCodg("wy");
        feeDetail.setBilgDrName("wy");


        // =============== 上传响应参数 ===============
        // payOrdId 支付订单号
        // payToken 支付token
        // cashierUrl 医保支付收银台h5地址  不对接收银台模式 忽略


        return null;
    }


    public InsuranceUploadRecipeDetailOutputMessage uploadRecipeDetail(InsuranceGetOrderReq req, InsuranceRep insuranceInfo, InsuranceCoreOutputMessage1101 insuranceCoreInfo) {


        // ============ 参数梳理 ===========

        // orgCodg 机构编码
        // orgId 电子凭证机构号
        // psnNo 人员编号 来源于 医保中心 医保核心系统 「1101」接口
        // insutype 险种类型 310:职工基本医疗保险 390:城乡居民基本医疗保险 320:公务员医疗补助 392:城乡居民大病保险 330:大额医疗费用补助 340:离休人员医疗保障 510:生育保险
        // medOrgOrd 医疗机构订单号  todo 使用药费订单号
        // rxCircFlag 电子处方流转标志  todo 确认是否需要
        // begntime 开始时间 挂号时间 格式： yyyy-MM-dd HH:mm:ss
        // idNo 患者证件号码
        // userName 用户姓名
        // idType 证件类别 居民身份证 01
        // ecToken 电子凭证授权
        // insuCode 就诊参保地行政区划  todo 确认是否需要 为空(null)时取用户电子凭证默认参保地
        // iptOtpNo 住院/门诊号  todo 确认是否可以用问诊单号
        // deptName 科室名称
        // caty 科别
        // medType 医疗类别 todo 确认互联网医院处方的医疗类别
        // feeType 费用类别  todo 确认互联网医院处方的费用类别 互联网医院问诊 05
        // medfeeSumamt 医疗费总额
        // acctUsedFlag 是否使用个人账户  todo 确认是否需要 药店上传费用时必填
        // diseCodg 疾病编码 todo 确认类型
        // diseName 疾病名称
        // psnSetlway 个人结算方式  todo 确认字典
        // chrgBchno 收费批次号
        // pubHospRfomFlag 公立医院改革标志

        // -diseinfoList 诊断或症状明细
        //      -diagType 诊断类型 1 西医主要诊断 2 西医其他诊断 3 中医主病诊断 4 中医主证诊断
        //      -diagSrtNo 诊断排序号
        //      -diagCode 诊断代码
        //      -diagName 诊断名称
        //      -diagDept 诊断科室
        //      -diseDorNo 诊断医生编码
        //      -diseDorName 诊断医生姓名
        //      -diagTime 诊断时间 yyyy-MM-dd HH:mm:ss
        //      -valiFlag 有效标志 0 无效 1 有效
        // -feedetailList 费用明细
        //      -feedetlSn 费用明细序号
        //      -psnNo 人员编号 来源于 医保中心 医保核心系统 「1101」接口
        //      -chrgBchno 收费批次号
        //      -diseCodg 疾病编码
        //      -rxno 处方号 todo 外购处方时，传入外购处方的处方号；非外购处方，传入医药机构处方号
        //      -rxCircFlag todo 外购处方标志 需要确认互联网医院的处方是否为外购处方
        //      -feeOcurTime 费用发生时间 yyyy-MM-dd HH:mm:ss
        //      -medListCodg 医疗目录编码 todo 确认互联网医院处方的医疗目录编码
        //      -medinsListCodg 医药机构目录编码 todo 确认互联网医院处方的医药机构目录编码
        //      -detItemFeeSumamt 明细项目费用总额
        //      -cnt 数量
        //      -pric 单价
        //      -bilgDeptCodg 开单科室编码
        //      -bilgDeptName 开单科室名称
        //      -bilgDrCodg 开单医生编码 todo 医生标准编码
        //      -bilgDrName 开单医师姓名
        //      -hospApprFlag 医院审批标志 1
        //      -medType 医疗类别 todo 确认互联网医院处方的医疗类别
        //      -medListName 医疗目录名称
        //      -medListSpc 医疗目录规格
        // payAuthNo 支付授权码
        // uldLatlnt 用户经纬度 定位失败情况下传0,0
        // mdtrtCertType 就诊凭证类型 02 身份证 01 电子凭证 todo 确认填身份证还是电子凭证


        // todo 校验必填项


        InsuranceUploadRecipeDetailInputMessage uploadData = new InsuranceUploadRecipeDetailInputMessage();
        // 医保核心系统患者基本信息
        InsuranceCoreOutputMessage1101.Baseinfo baseinfo = insuranceCoreInfo.getBaseinfo();
        //  医保核心系统保信息
        InsuranceCoreOutputMessage1101.Insuinfo insuinfo = insuranceCoreInfo.getInsuinfo();
        // 费用明细
        List<FeeDetail> feedetailList = new ArrayList<>();
        // 诊断或状态明细
        List<DiseInfo> diseinfoList = new ArrayList<>();


        uploadData.setOrgCodg(fixmedinsCode);
        //        uploadData.setOrgId();
        uploadData.setPsnNo(baseinfo.getPsnNo());
        uploadData.setIdType(insuinfo.getInsutype());
        uploadData.setMedOrgOrd(req.getMedOrgOrd());
        // 电子处方流转标志  todo 目前河南省并未测试过电子处方线上医保支付
        uploadData.setRxCircFlag("0");
        uploadData.setBegntime(DateUtils.format(new Date(), DateUtils.PATTERN_DATE_TIME));
        uploadData.setIdNo(baseinfo.getCertno());
        uploadData.setUserName(baseinfo.getPsnName());
        uploadData.setIdType("01");
        uploadData.setEcToken(req.getAuthCode());
        uploadData.setInsuCode(mdtrtareaAdmvs);
        uploadData.setIptOtpNo(req.getPlatformOrderId());
        // 字典 dept
        uploadData.setDeptName("预防保健科");
        uploadData.setCaty("A01");
        uploadData.setMedType("91");
        uploadData.setFeeType("05");
        uploadData.setMedfeeSumamt(new BigDecimal("0.01"));
        //        uploadData.setAcctUsedFlag("0");
        uploadData.setDiseCodg("A01.001");
        uploadData.setDiseName("感冒");
        uploadData.setPsnSetlway("01");
        // 公立医院内容 todo   疑问点 互联网医院是否为公立医院 公立医院改革标志 如何传参
        uploadData.setChrgBchno(req.getChrgBchno());
        uploadData.setDiseinfoList(diseinfoList);
        uploadData.setFeedetailList(feedetailList);

        // ========组装诊断数据========
        DiseInfo diseInfo = new DiseInfo();
        diseinfoList.add(diseInfo);
        // 西医诊断
        diseInfo.setDiagType("1");
        // todo 确认诊断排序号
        diseInfo.setDiagSrtNo(1);
        diseInfo.setDiagCode("A01.001");
        diseInfo.setDiagName("感冒");
        diseInfo.setDiagDept("预防保健科");
        diseInfo.setDiseDorNo("wy");
        diseInfo.setDiseDorName("wy");
        diseInfo.setDiagTime(DateUtils.format(new Date(), DateUtils.PATTERN_DATE_TIME));
        diseInfo.setValiFlag("1");

        //========组装费用数据========
        FeeDetail feeDetail = new FeeDetail();

        feedetailList.add(feeDetail);
        // 如果是诊疗 传问诊单号
        feeDetail.setFeedetlSn("10086");
        feeDetail.setMdtrtId(req.getMdtrtId());
        feeDetail.setPsnNo(baseinfo.getPsnNo());
        feeDetail.setChrgBchno(uploadData.getChrgBchno());
        // 同一收费批次号病种编号必须一致 todo 确认 多药品多问题
        feeDetail.setDiseCodg("A01.001");
        feeDetail.setRxno("100866");
        feeDetail.setRxCircFlag("0");
        feeDetail.setFeeOcurTime(DateUtils.format(new Date(), DateUtils.PATTERN_DATE_TIME));
        // 医疗目录编码 todo 确认互联网医院处方的医疗目录编码
        feeDetail.setMedListCodg("A01.001");
        // 医药机构目录编码 todo 确认互联网医院处方的医药机构目录编码
        feeDetail.setMedinsListCodg("A01.001");
        feeDetail.setDetItemFeeSumamt(new BigDecimal("0.01"));
        feeDetail.setCnt(new BigDecimal("1"));
        feeDetail.setPric(new BigDecimal("0.01"));
        feeDetail.setBilgDeptCodg("A01.001");
        feeDetail.setBilgDeptName("预防保健科");
        feeDetail.setBilgDrCodg("wy");
        feeDetail.setBilgDrName("wy");
        feeDetail.setHospApprFlag("1");
        // todo 确认互联网医院处方的医疗类别
        feeDetail.setMedType("医疗类别");
        feeDetail.setMedListName("医疗目录名称");
        feeDetail.setMedListSpc("医疗目录规格");


        String result = HttpUtil.postByJson(uploadRecipeDetailPath, JSONObject.toJSONString(uploadData));
        log.info("uploadRecipeDetailPath: {}", result);
        InsuranceMiddleGroundOutputMessage outputMessage = JSON.parseObject(result, InsuranceMiddleGroundOutputMessage.class);

        // =============== 上传响应参数 ===============
        // payOrdId 支付订单号
        // payToken 支付token
        // cashierUrl 医保支付收银台h5地址  不对接收银台模式 忽略
        if (outputMessage.getCode() != null) {
            // 解密outputMessage
            decryptMiddleGroundOutputMessage(outputMessage);


            return JSON.parseObject(outputMessage.getData(), InsuranceUploadRecipeDetailOutputMessage.class);
        }


        return null;
    }


    private void decryptMiddleGroundOutputMessage(InsuranceMiddleGroundOutputMessage outputMessage) {
        // 解密outputMessage todo
        String data = outputMessage.getEncData();
        outputMessage.setData(data);
    }


    /**
     * 省医保平台「6202」支付下单
     *
     * @return
     */
    public InsurancePayOrderOutputMessage payOrder(InsuranceGetOrderReq req, InsuranceRep insuranceRep, InsuranceUploadRecipeDetailOutputMessage uploadRecipeDetail) {

        // ============ 参数梳理 ===========
        // payAuthNo 支付授权码
        // payOrdId 待支付订单号 费用上传接口
        // payToken 支付订单对应的token 费用上传接口
        // orgCodg 定点机构编码
        // orgBizSer 业务流水号 每次请求唯一
        // chrgBchno 收费批次号 费用上传一致
        // feeType 费用类别 与费用上传一致
        // mdtrtId 就诊事件ID 与费用上传一致

        InsurancePayOrderInputMessage payOrder = new InsurancePayOrderInputMessage();

        payOrder.setPayAuthNo(insuranceRep.getPayAuthNo());
        payOrder.setPayOrdId(uploadRecipeDetail.getPayOrdId());
        payOrder.setPayToken(uploadRecipeDetail.getPayToken());
        payOrder.setOrgCodg(fixmedinsCode);
        payOrder.setOrgBizSer(req.getOrgBizSer());
        payOrder.setChrgBchno(req.getChrgBchno());


        String result = HttpUtil.postByJson(payOrderPath, JSONObject.toJSONString(payOrder));
        log.info("payOrder: {}", result);
        InsuranceMiddleGroundOutputMessage outputMessage = JSON.parseObject(result, InsuranceMiddleGroundOutputMessage.class);


        // =============== 下单响应参数 ===============
        // payOrdId 支付订单号
        // ordStas 支付订单状态 0 已保存 1 预结算完成 2 结算中 3 自费完成 4 医保支付完成 5 院内结算完成 6 结算完成
        // feeSumamt 费用总额
        // ownPayAmt 现金支付
        // psnAcctPay 个人账户支出
        // fundPay 医保基金支付
        if (outputMessage.getCode() != null) {
            // 解密outputMessage
            decryptMiddleGroundOutputMessage(outputMessage);

            return JSON.parseObject(outputMessage.getData(), InsurancePayOrderOutputMessage.class);
        }



        return null;


    }

    /**
     * 获取用户医保信息
     * 目前测试由配置文件获取
     * @param req 请求参数
     * @return 返回用户医保信息
     */
    @SneakyThrows
    public InsuranceRep getInsuranceInfo(InsuranceReq req) {
//        Integer patientUserId = TokenUtil.getPatientUserId();
//        PatientUser patientUser = patientUserMapper.selectById(patientUserId);

        UserQueryReq queryReq = new UserQueryReq();
        queryReq.setOpenid("");
        queryReq.setQrcode(req.getQrcode());
        String pathUrl = String.format(getInsuranceAPI, partnerId);
        String s = postData("mipuserquery", "userQuery", pathUrl, partnerId, queryReq.toString());
        log.info("getInsuranceInfo: {}", s);
        return JSON.parseObject(s, InsuranceRep.class);

    }


    /**
     * 微信支付平台统一下单
     *
     * @return
     */
    @SneakyThrows
    public UnifiedOrderRep unifiedOrder(UnifiedOrderReq req) {

        InsuranceAccessTokenRep accessToken = req.getAccessToken();
        // 医保下单信息
        InsurancePayOrderOutputMessage order = req.getInsurancePayOrder();


        // ========== 参数梳理 ==========
        WechatUnifiedOrderReq orderReq = new WechatUnifiedOrderReq();
        orderReq.setOrderType(req.getOrderType());
        orderReq.setAppid(appid);
        orderReq.setMchId(mchId);
        orderReq.setSubMchId(subMchId);
        orderReq.setOpenid(req.getOpenId());
        orderReq.setHospOutTradeNo(UUID.randomUUID().toString());
        orderReq.setHospitalName(fixmedinsName);
        orderReq.setNonceStr(UUID.randomUUID().toString().replace("-", ""));
        // 总额 转换为分
        orderReq.setTotalFee(Integer.toString(order.getFeeSumamt().multiply(new BigDecimal("100")).intValue()));
        // 现金支付 转换为分
        orderReq.setCashFee(Integer.toString(order.getOwnPayAmt().multiply(new BigDecimal("100")).intValue()));
        // 减免、 运费 todo .......后续可拓展
        orderReq.setCashAddFee("0");
        //  当医保局返回的预结算金额与医院上传的金额不一致时，此字段为0则直接报错，为1则以医保局金额为准
        orderReq.setAllowFeeChange("0");
        orderReq.setSpbillCreateIp(req.getSpbillCreateIp());
        orderReq.setNotifyUrl("");
        orderReq.setBody("医保支付");
        // todo 确认跳转地址
        orderReq.setReturnUrl("");
        // 当订单要走医保支付时，
        //（1）cash_fee>0, paytype填3
        //（2）cash_fee=0, paytype填2
        Integer payType = orderReq.getCashFee().compareTo("0") == 0 ? 2 : 3;
        orderReq.setPayType(payType.toString());
        orderReq.setCityId(cityId);
        orderReq.setConsumeType("0");
        orderReq.setInsuranceFee(Integer.toString(order.getFundPay().multiply(new BigDecimal("100")).intValue()));
        // sfz 1 身份证
        orderReq.setUserCardType("1");
        // todo MD5处理
        orderReq.setUserCardNo(req.getUserCardNo());
        orderReq.setUserName(req.getUserName());
        // is_dept is_dept=4 医保电子凭证支付；
        orderReq.setIsDept("4");
        orderReq.setSerialNo(UUID.randomUUID().toString());
        orderReq.setOrgNo(orgCodg);
        orderReq.setGmtOutCreate(DateUtils.format(new Date(), DateUtils.PATTERN_YYYYMMDDHHMMSS));
        orderReq.setRequestContent(order.getRequestContent());
        orderReq.setSign("");

        String url = String.format(unifiedOrderAPI, accessToken.getAccessToken());

        String s = postData("mipunifiedorder", "unifiedOrder", url, agentappid, orderReq.toString());


        return null;
    }

    /**
     * 获取当前HealthInsuranceManager的所有属性
     *
     * @return 返回当前HealthInsuranceManager的所有属性
     */
    public Object getInsuranceConfig() {
        // 返回当前HealthInsuranceManager的所有属性

        Map<String, String> map = new HashMap<>();
        map.put("partnerId", partnerId);
        map.put("partnerSecret", partnerSecret);
        map.put("agentappid", agentappid);
        map.put("agentsecret", agentsecret);
        map.put("appid", appid);
        map.put("cityCode", cityCode);
        map.put("channel", channel);
        map.put("redirectUrl", redirectUrl);
        map.put("orgChnlCrtfCodg", orgChnlCrtfCodg);
        map.put("orgCodg", orgCodg);
        map.put("bizType", bizType);
        map.put("orgAppId", orgAppId);

        return JSON.toJSON(map, JSONWriter.Feature.WriteNulls);
    }


    // ======================  签名方法 调用微信医保平台接口时使用 =====================


    private String bytesToHex(byte[] bytes) {
        char[] hexChars = new char[bytes.length * 2];
        for (int j = 0; j < bytes.length; j++) {
            int v = bytes[j] & 0Xff;
            hexChars[j * 2] = hexArray[v >>> 4];
            hexChars[j * 2 + 1] = hexArray[v & 0x0F];
        }
        return new String(hexChars);
    }

    private String createSignature(String partnerSecret, String partnerId, String timestamp)
            throws UnsupportedEncodingException, NoSuchAlgorithmException, InvalidKeyException {
        Mac sha256_HMAC = Mac.getInstance("HmacSHA256");
        SecretKeySpec secret_key = new SecretKeySpec(partnerSecret.getBytes("UTF-8"), "HmacSHA256");
        sha256_HMAC.init(secret_key);
        return bytesToHex(sha256_HMAC.doFinal((partnerId + timestamp).getBytes("UTF-8")));
    }


    private String postData(String businessName, String funcName, String urlPath, String partnerId, String data)
            throws IOException, NoSuchAlgorithmException, InvalidKeyException {
        HttpURLConnection conn = null;
        URL url = new URL(urlPath);
        conn = (HttpURLConnection) url.openConnection();
        conn.setDoOutput(true);
        conn.setDoInput(true);
        conn.setRequestMethod("POST");
        conn.setUseCaches(false);
        conn.setInstanceFollowRedirects(true);
        conn.setRequestProperty("Content-Type", "application/json");
        conn.setRequestProperty("Accept", "application/json");
        String timestamp = Long.toString(System.currentTimeMillis());
        String signature = createSignature(partnerSecret, partnerId, timestamp);
        String requestId = UUID.randomUUID().toString().replaceAll("-", "");
        conn.setRequestProperty("god-portal-timestamp", timestamp);
        conn.setRequestProperty("god-portal-signature", signature);
        conn.setRequestProperty("god-portal-request-id", requestId);
        conn.connect();
        OutputStream os = conn.getOutputStream();
        os.write(data.getBytes());
        os.flush();
        os.close();
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        InputStream inputStream = conn.getInputStream();
        byte[] bytes = new byte[1024];
        int readBytes;
        while ((readBytes = inputStream.read(bytes)) != -1) {
            byteArrayOutputStream.write(bytes, 0, readBytes);
        }
        bytes = byteArrayOutputStream.toByteArray();
        inputStream.close();
        conn.disconnect();
        return new String(bytes, "utf-8");
    }


}
