package com.szyx.tcm.supervision.util;


import com.szyx.tcm.supervision.enums.BannerEnum;
import com.szyx.tcm.supervision.enums.WeekDayEnum;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.dromara.hutool.core.date.DateUtil;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * @author chenjie
 */
public class DateUtils extends org.apache.commons.lang3.time.DateUtils {

    public static final String PATTERN_DATE_TIME = "yyyy-MM-dd HH:mm:ss";
    public static final String PATTERN_DATE_TIME_YEAR = "yyyy-MM-dd HH:mm";
    public static final String PATTERN_DATE = "yyyy-MM-dd";
    public static final String PATTERN_DATE_SIDE= "yyyyMMdd";
    public static final String PATTERN_DATE_HOUR = "HH:mm";
    public static final String PATTERN_DATE_HOUR_MS = "HH:mm:ss";
    public static final String PATTERN_YYYYMMDDHHMMSSSSS = "yyyyMMddHHmmssSSS";
    public static final String PATTERN_YYYYMMDDHHMMSS = "yyyyMMddHHmmss";
    public static final String PATTERN_DATE_YEAR = "yyyy";
    public static final String UTC_WITH_ZONE_OFFSET_PATTERN = "yyyy-MM-dd'T'HH:mm:ss+08:00";

    public static final String PATTERN_MONTH_DATE = "MM.dd";

    public static final String PATTERN_YEAR_DATE = "yyyy.MM.dd";
    public static Date parseDate(String str) {
        try {
            return parseDate(str, PATTERN_DATE_TIME);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }
    public static Date parseDateHours(String str) {
        try {
            return parseDate(str, PATTERN_DATE_HOUR_MS);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }


    public static String format(Date date) {
        return DateFormatUtils.format(date, PATTERN_DATE_TIME);
    }

    public static String formatString(Date date) {
        return DateFormatUtils.format(date, PATTERN_DATE);
    }

    public static String formatStringToYear(Date date) {
        return DateFormatUtils.format(date, PATTERN_DATE_YEAR);
    }

    public static String formatString1(String date) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(PATTERN_DATE);
        return formatString(simpleDateFormat.parse(date));
    }

    public static String formatStringYear(Date date){
      return new SimpleDateFormat("yyyy年MM月dd日").format(date);
    }

    public static String formatYear(Date date) {
        return DateFormatUtils.format(date, PATTERN_DATE_TIME_YEAR);
    }

    public static String format(Date date, String pattern) {
        return DateFormatUtils.format(date, pattern);
    }

    public static Date parse(String dateStr) throws Exception {
        SimpleDateFormat formatter = new SimpleDateFormat(PATTERN_DATE);
        return formatter.parse(dateStr);
    }

    public static Date parse(String dateStr, String pattern_date) throws Exception {
        SimpleDateFormat formatter = new SimpleDateFormat(pattern_date);
        return formatter.parse(dateStr);
    }

    /**
     * 计算当前日期与{@code endDate}的间隔天数
     *
     * @param date
     * @return 间隔天数
     */
    public static long until(Date date) {

        LocalDate endDate = dateLocalDate(date);
        return LocalDate.now().until(endDate, ChronoUnit.DAYS);
    }

    /**
     * 计算日期{@code startDate}与{@code endDate}的间隔天数
     *
     * @param startDate
     * @param endDate
     * @return 间隔天数
     */
    public static long until(LocalDate startDate, LocalDate endDate) {
        return startDate.until(endDate, ChronoUnit.DAYS);
    }

    /**
     * Date转LocalDate
     *
     * @param date
     */
    public static LocalDate dateLocalDate(Date date) {
        if (null == date) {
            return null;
        }
        return date.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
    }

    public static String getDatePoor(Date endDate, Date nowDate, Integer status) {

        long nd = 1000 * 24 * 60 * 60;//每天毫秒数

        long nh = 1000 * 60 * 60;//每小时毫秒数

        long nm = 1000 * 60;//每分钟毫秒数

        long diff = endDate.getTime() - nowDate.getTime(); // 获得两个时间的毫秒时间差异

        long day = diff / nd;   // 计算差多少天

        long hour = diff % nd / nh; // 计算差多少小时

        long min = diff % nd % nh / nm;  // 计算差多少分钟
        if (status == 1) {
            if (day > 0) {
                return day + "天" + hour + "小时";
            } else {
                return hour + "小时";
            }
        } else {
            if (day == 0 && hour == 0) {
                return String.valueOf(min);
            } else {
                return String.valueOf(100);
            }
        }
    }

    /**
     * LocalDate转Date
     *
     * @param localDate
     * @return
     */
    public static Date localDate2Date(LocalDate localDate) {
        if (null == localDate) {
            return null;
        }
        ZonedDateTime zonedDateTime = localDate.atStartOfDay(ZoneId.systemDefault());
        return Date.from(zonedDateTime.toInstant());
    }

    /**
     * Date转LocalDate
     *
     * @param date
     */
    public static LocalDate date2LocalDate(Date date) {
        if (null == date) {
            return null;
        }
        return date.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
    }

    /**
     * LocalDate 转string
     *
     * @param localDate
     * @return
     */
    public static String localDate2String(LocalDate localDate) {
        if (null == localDate) {
            return null;
        }
        DateTimeFormatter fmt = DateTimeFormatter.ofPattern(PATTERN_DATE);
        return localDate.format(fmt);
    }

    public static String localDateTimeToString(LocalDateTime localDate, String patternDate) {
        if (null == localDate) {
            return null;
        }

        DateTimeFormatter fmt = DateTimeFormatter.ofPattern(patternDate);
        return localDate.format(fmt);
    }

    /**
     * 计算当前日期与{@code endDate}的间隔天数
     *
     * @param endDate
     * @return 间隔天数
     */
    static long until(LocalDate endDate) {
        return LocalDate.now().until(endDate, ChronoUnit.DAYS);
    }

    /**
     * 计算日期{@code startDate}与{@code endDate}的间隔天数
     *
     * @param startDate
     * @param endDate
     * @return 间隔天数
     */
    static long untilDiffer(LocalDate startDate, LocalDate endDate) {
        return startDate.until(endDate, ChronoUnit.DAYS);
    }

    public static String betweenTwoTimes(LocalDate startDate, LocalDate endDate) {
        Period p = Period.between(endDate, startDate);
        return p.getYears() + "年" + p.getMonths() + "月" + p.getDays() + "日";
    }

    public final static Map<Character, Integer> digit = new HashMap<>();
    public final static Map<Character, Integer> position = new HashMap<>();

    static {
        digit.put('零', 0);
        digit.put('一', 1);
        digit.put('二', 2);
        digit.put('三', 3);
        digit.put('四', 4);
        digit.put('五', 5);
        digit.put('六', 6);
        digit.put('七', 7);
        digit.put('八', 8);
        digit.put('九', 9);
        position.put('十', 1);
        position.put('百', 2);
        position.put('千', 3);
    }

    public static int s2(String num) {
        //一共八位
        char[] nums = new char[8];
        int p = 7, now = 7;
        char[] c = num.toCharArray();
        /**
         * 从右到左逐个字符解析
         * 通过p控制万位,pp控制万以下的单位,now记录当前单位
         */
        for (int i = c.length - 1; i >= 0; i--) {
            if (c[i] == '万') {
                now = p = 3;
            } else {
                int d = digit.getOrDefault(c[i], -1);
                if (d == 0) {
                    continue;
                }
                if (d == -1) {
                    int pp = position.getOrDefault(c[i], -1);
                    if (pp == -1) {
                        throw new RuntimeException("\"" + num + "\"中存在无法解析的字符: " + i + ":" + c[i]);
                    } else {
                        now = p - pp;
                        //允许"一十"省略"一",即"十"
                        if (now == 6) {
                            nums[now] = 1;
                        }
                    }
                } else {
                    nums[now] = (char)d;
                }
            }
        }
        for (int i = 0; i < nums.length; i++) {
            nums[i] = (char)(nums[i] + '0');
        }
        return Integer.parseInt(new String(nums));
    }

    /**
     * 把 Thu Dec 14 00:00:00 CST 2017 转换成自己想要的格式
     *
     * @param date
     * @param pattern
     * @return
     */

    public static String dateStringFormat(String date, String pattern) {
        if (date == null || pattern == null) {
            return null;
        }
        try {
            Date formDate = new SimpleDateFormat("MMM dd yyyy", Locale.US).parse(date);
            return new SimpleDateFormat(pattern, Locale.CHINA).format(formDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static Date dateFormatConvert(String Date, String format, String newFormat) {
        try {
            SimpleDateFormat newSDF = new SimpleDateFormat(newFormat);
            SimpleDateFormat oldSDF = new SimpleDateFormat(format);
            Date inDate = oldSDF.parse(Date);
            String outDate = newSDF.format(inDate);
            return newSDF.parse(outDate);
        } catch (Exception e) {
            return null;
        }
    }

    public static Date getDate(String sFormat, String sDate) {
        SimpleDateFormat sdf = new SimpleDateFormat(sFormat);
        Date d = null;
        try {
            sdf.setLenient(false);
            d = sdf.parse(sDate);
        } catch (Exception e) {
            return null;
        }
        return d;
    }

    public static String getDate(String sDate) {
        String[] sa =
            {"yyyy-MM-dd", "yyyyMMdd", "yyyy.MM.dd", "yyyy/MM/dd", "yyyy年MM月dd日", "dd/MM/yyyy", "d/M/yyyy", "MM-dd-yyyy", "dd-MM-yyyy", "MM/yyyy", "yyyy--M-dd",
                "yyyy-MM", "yyyy"};

        for (String s : sa) {
            Date d = getDate(s, sDate);
            if (d != null) {
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
                return simpleDateFormat.format(d);
            }
        }
        return null;
    }

    /**
     * 把 mm/dd/yyyy 转换成
     *
     * @param date
     * @return
     */

    public static String dateStringFormat(String date) {
        if (StringUtils.isNotBlank(date)) {
            String[] split = date.split("/");
            if (split.length == 3 && split[2].length() == 4) {
                return split[2] + "-" + split[0] + "-" + split[1];
            }
            return date;
        }
        return "";
    }

    public static String dataToUpper(String dateString) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date dateTime = null;
        try {
            dateTime = simpleDateFormat.parse(dateString);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Calendar ca = Calendar.getInstance();
        ca.setTime(dateTime);
        int year = ca.get(Calendar.YEAR);
        int month = ca.get(Calendar.MONTH) + 1;
        int day = ca.get(Calendar.DAY_OF_MONTH);
        return numToUpper(year) + "年" + monthToUppder(month) + "月" + dayToUppder(day) + "日";
    }

    // 将数字转化为大写（字体格式可自己定义）
    public static String numToUpper(int num) {
        //String u[] = {"零","壹","贰","叁","肆","伍","陆","柒","捌","玖"};
        String[] u = {"零","一","二","三","四","五","六","七","八","九"};
        char[] str = String.valueOf(num).toCharArray();
        String rstr = "";
        for (int i = 0; i < str.length; i++) {
            rstr = rstr + u[Integer.parseInt(String.valueOf(str[i]))];
        }
        return rstr;
    }

    // 月转化为大写
    public static String monthToUppder(int month) {
        if(month < 10) {
            return numToUpper(month);
        } else if(month == 10){
            return "十";
        } else {
            return "十" + numToUpper(month - 10);
        }
    }

    // 日转化为大写
    public static String dayToUppder(int day) {
        if(day < 20) {
            return monthToUppder(day);
        } else {
            char[] str = String.valueOf(day).toCharArray();
            if(str[1] == '0') {
                return numToUpper(Integer.parseInt(String.valueOf(str[0]))) + "十";
            }else {
                return numToUpper(Integer.parseInt(String.valueOf(str[0]))) + "十" + numToUpper(Integer.parseInt(String.valueOf(str[1])));
            }
        }
    }

    /**
     * 判断当前日期是否在指定日期内
     * @param start
     * @param end
     * @return
     */
    public static Integer beforeTimeCalendar(Date start, Date end) {

        LocalDate startDate = dateToLocalDate(start);

        LocalDate endDate = dateToLocalDate(end);
        // 当前日期
        LocalDate today = LocalDate.now();
        if (today.isAfter(startDate) && today.isBefore(endDate)) {
            return BannerEnum.PUBLISH.getCode();
        } else {
            if(today.isBefore(startDate)) {
                return BannerEnum.EXPIRED.getCode();
            }
            if(today.isAfter(endDate)) {
                return BannerEnum.RELEASED.getCode();
            }
        }
        return BannerEnum.PUBLISH.getCode();
    }

    /**
     * JAVA计算两个日期相差多少天
     * @param date2
     * @param date2
     * @return
     */
    public static int daysBetween(Date date1,Date date2)  {
        int count = 0;
        try {
           if(date1 == null || date2 == null ) {
               return 0;
           }
           count = Math.abs((int) ((date1.getTime() - date2.getTime()) / (1000*3600*24)));
        } catch (Exception e) {
             e.printStackTrace();
        }
        return count;
    }


    /**
     * Date 转为 LocalDate
     *
     * @param  date
     * @return LocalDate;
     */
    public static LocalDate dateToLocalDate(Date date) {
        return date.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
    }

    @SneakyThrows
    public static Date formatDate(Date date, String pattern) {
        if (pattern == null) {
            pattern = "yyyy-MM-dd";
        }

        SimpleDateFormat formatter = new SimpleDateFormat(pattern);
        DateFormat dateFormat = new SimpleDateFormat(pattern);
        Date formatDate = dateFormat.parse(formatter.format(date));
        return formatDate;
    }

    /**
     * 获取年龄
     * @param birthday
     * @return
     */
    public static Integer getAge(Date birthday) {
        int compare = DateUtil.compare(birthday, new Date());
        if (compare > 0) {
            //如果出生日期大于当前时间,返回0
            return 0;
        } else {
            return DateUtil.ageOfNow(birthday);
        }
    }


    /**
     * 根据日期获得星期
     * @param date
     * @return
     */
    public static String getWeekOfDate(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int intWeek = calendar.get(Calendar.DAY_OF_WEEK);
        return WeekDayEnum.getByCode(intWeek);
    }

    public static Date getWholeTime(Date date,int addDay){
        if (date == null) {
            date = new Date();
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, addDay);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    public static Integer dayForWeeks(Date date)  {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int  dayForWeek = 0 ;
        if (c.get(Calendar.DAY_OF_WEEK) == 1 ){
            dayForWeek = 7 ;
        }else {
            dayForWeek = c.get(Calendar.DAY_OF_WEEK) - 1 ;
        }
        return  dayForWeek;
    }

    public static int getCompareResult(Date date, Date targetDate) {
        return formatDate(date, DateUtils.PATTERN_DATE_TIME_YEAR).compareTo(DateUtils.formatDate(targetDate, DateUtils.PATTERN_DATE_TIME_YEAR));
    }

    /** 
     * 方法注释: <br>
     * 〈时间增加一个小时〉
     * @param	hour	
     * @return java.util.Date
     * @author cj 🤡
     * @date 2024/1/17 19:15
     */
    public static Date timeSubtraction(Integer hour){
        // 当前时间
        LocalDateTime currentDateTime = LocalDateTime.now();

        // 添加小时
        LocalDateTime newDateTime = currentDateTime.plusHours(hour);

        Date date = Date.from(newDateTime.atZone(ZoneId.systemDefault()).toInstant());


        return date;
    }

    public static void main(String[] args) {
        Date startTime= DateUtils.formatDate(new Date(), DateUtils.PATTERN_DATE_HOUR_MS);
        startTime.setDate(new Date().getDate());
        Date start= DateUtils.parseDateHours("16:30:00");

        Date end= DateUtils.parseDateHours("23:30:00");
        if ((start.after(startTime))) {
            System.out.println(true);
        }

        if ((end.before(startTime))) {
            System.out.println(true);
        }
    }
}


