package com.szyx.tcm.supervision.util;

import com.tencent.mip.DataHandler;
import org.dromara.hutool.core.util.RandomUtil;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.UUID;

/*******************************************************************
 * <pre></pre>
 * @文件名称： InsuranceWechatUtil.java
 * @包 路  径： com.szyx.tcm.common.utils
 * @Copyright：wy (C) 2024 *
 * @Description: 医保支付参数封装工具类
 * @Version: V1.0
 * @Author： wy
 * @Date： 2024/7/22 13:42
 * @Modify：
 */
public class InsuranceWechatUtil {

    // ================= 医保核心系统有关方法 ===========================

    /**
     * 医保核心系统-生成发送报文ID
     *
     * @param Code 医院机构编码 K开头
     */
    public static String getMsgId(String Code) {

        // 时间类型 yyyyMMddHHmmss
        // 顺序号 待定 todo
        if (Code == null) {
            Code = "";
        }
        return Code + DateUtils.format(new Date(), DateUtils.PATTERN_YYYYMMDDHHMMSS) + RandomUtil.randomNumbers(4);
    }


    // ================= 医保支付中台有关方法 ===========================

    /**
     * 根据公私钥和appID生成请求报文
     */
    public static String getReqMessageForInsuranceCore(String appid, String appSecret, String publicKey,String privateKey,String plainJSON) throws Exception {
        DataHandler dataHandler =  DataHandler.newInstance(appid,appSecret,publicKey,privateKey);
        dataHandler.setVersion("2.0.1");

        return dataHandler.buildReqData(plainJSON);

    }






    // ================= 微信医保支付平台有关方法 ===========================

    /**
     *
     */
    private final static char[] hexArray = "0123456789abcdef".toCharArray();

    //签名工具 获取信息服务接口
    private static String createSignature(String partnerSecret, String partnerId, String timestamp)
            throws UnsupportedEncodingException, NoSuchAlgorithmException, InvalidKeyException {
        Mac sha256_HMAC = Mac.getInstance("HmacSHA256");
        SecretKeySpec secret_key = new SecretKeySpec(partnerSecret.getBytes("UTF-8"), "HmacSHA256");
        sha256_HMAC.init(secret_key);
        return bytesToHex(sha256_HMAC.doFinal((partnerId + timestamp).getBytes("UTF-8")));
    }

    private static String bytesToHex(byte[] bytes) {
        char[] hexChars = new char[bytes.length * 2];
        for (int j = 0; j < bytes.length; j++) {
            int v = bytes[j] & 0Xff;
            hexChars[j * 2] = hexArray[v >>> 4];
            hexChars[j * 2 + 1] = hexArray[v & 0x0F];
        }
        return new String(hexChars);
    }

    /**
     * 发送请求
     *  微信免密授权请求
     * @param businessName 自定义业务名称
     * @param funcName     自定义功能名称
     * @param urlPath      请求路径
     * @param partnerId    合作方ID
     * @param data         请求数据
     * @return 返回结果
     * @throws IOException
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeyException
     */
    public static String postData(String businessName, String funcName, String partnerSecret, String urlPath, String partnerId, String data)
            throws IOException, NoSuchAlgorithmException, InvalidKeyException {
        HttpURLConnection conn = null;
        URL url = new URL(urlPath);
        conn = (HttpURLConnection) url.openConnection();
        conn.setDoOutput(true);
        conn.setDoInput(true);
        conn.setRequestMethod("POST");
        conn.setUseCaches(false);
        conn.setInstanceFollowRedirects(true);
        conn.setRequestProperty("Content-Type", "application/json");
        conn.setRequestProperty("Accept", "application/json");
        String timestamp = Long.toString(System.currentTimeMillis());
        String signature = createSignature(partnerSecret, partnerId, timestamp);
        String requestId = UUID.randomUUID().toString().replaceAll("-", "");
        conn.setRequestProperty("god-portal-timestamp", timestamp);
        conn.setRequestProperty("god-portal-signature", signature);
        conn.setRequestProperty("god-portal-request-id", requestId);
        conn.connect();
        OutputStream os = conn.getOutputStream();
        os.write(data.getBytes());
        os.flush();
        os.close();
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        InputStream inputStream = conn.getInputStream();
        byte[] bytes = new byte[1024];
        int readBytes;
        while ((readBytes = inputStream.read(bytes)) != -1) {
            byteArrayOutputStream.write(bytes, 0, readBytes);
        }
        bytes = byteArrayOutputStream.toByteArray();
        inputStream.close();
        conn.disconnect();
        return new String(bytes, "utf-8");
    }


    public static void main(String[] args) {
        // 收到的1101报文
        String s = "{\n" +
                "  \"infcode\": \"1\",\n" +
                "  \"inf_refmsgid\": \"000000202001041235391234567890\",\n" +
                "  \"refmsg_time\": \"20200201133411352\",\n" +
                "  \"respond_time\": \"20200202133731456\",\n" +
                "  \"err_msg\": \"\",\n" +
                "  \"output\": {\n" +
                "    \"baseinfo\": {\n" +
                "      \"psn_no\": \"131000202001001\",\n" +
                "      \"psn_cert_type\": \"2\",\n" +
                "      \"certno\": \"510000202001010000\",\n" +
                "      \"psn_name\": \"李四\",\n" +
                "      \"gend\": \"1\",\n" +
                "      \"naty\": \"01\",\n" +
                "      \"brdy\": \"2020-01-01\",\n" +
                "      \"age\": 18\n" +
                "    },\n" +
                "    \"insuinfo\": {\n" +
                "      \"psn_insu_rlts_id\": \"133241523001001\",\n" +
                "      \"balc\": 5000,\n" +
                "      \"insutype\": \"310\",\n" +
                "      \"psn_type\": \"1001\",\n" +
                "      \"cvlserv_flag\": \"0\",\n" +
                "      \"insu_admdvs\": \"131002\",\n" +
                "      \"emp_name\": \"测试单位\"\n" +
                "    },\n" +
                "    \"idetinfo\": [\n" +
                "      {\n" +
                "        \"psn_idet_type\": \"1\",\n" +
                "        \"psn_type_lv\": \"1\",\n" +
                "        \"memo\": \"\",\n" +
                "        \"begntime\": \"2020-01-01 00:00:00\",\n" +
                "        \"endtime\": \"\"\n" +
                "      },\n" +
                "      {\n" +
                "        \"psn_idet_type\": \"2\",\n" +
                "        \"psn_type_lv\": \"1\",\n" +
                "        \"memo\": \"\",\n" +
                "        \"begntime\": \"2020-01-01 00:00:00\",\n" +
                "        \"endtime\": \"\"\n" +
                "      }\n" +
                "    ]\n" +
                "  }\n" +
                "}";


    }


}
